#!/usr/bin/env python3

import rospy
from geometry_msgs.msg import Twist
from std_msgs.msg import Bool
from sensor_msgs.msg import Joy
from std_srvs.srv import SetBool
from mccd_msgs.msg import Status
import time
import curses

############# DUALSHOCK 3 ##############

coefAng = -1
coefLin = 1

# BUTTONS

B_TRIG = 2
B_CIRC = 1
B_SQUARE = 3
B_CROSS = 0

B_L1 = 4
B_R1 = 5

B_L2 = 6
B_R2 = 7

B_SEL = 8
B_PS = 10
B_START = 9

B_LJOYC = 11
B_RJOYC = 12

B_UP = 13
B_DOWN = 14
B_LEFT = 15
B_RIGHT = 16

# AXES

A_LJOYH = 0
A_LJOYV = 1
A_L2 = 2

A_RJOYH = 3
A_RJOYV = 4
A_R2 = 5

########################################


class JoyTeleopNode():

    def joyCallback(self, data):

        if (data.buttons[B_START]):
            # Toggle PWM
            if self.status == Status.POWER_ON:
                self.enablePwm(False)
            else:
                self.enablePwm(True)

        if (data.buttons[B_PS]):
            # shutdown teleop
            self.forceQuit = True

        lin_vel = data.axes[A_LJOYV] * coefLin
        ang_vel = data.axes[A_LJOYH] * coefAng

        if lin_vel != self.lin_speed_cmd or ang_vel != self.ang_speed_cmd:
            self.lin_speed_cmd = lin_vel
            self.ang_speed_cmd = ang_vel
            self.update_info = True

    def __init__(self, stdscr):
        self.lin_speed_cmd = 0
        self.ang_speed_cmd = 0

        self.status = None

        self.update_info = False

        self.forceQuit = False
        self.publisher = rospy.Publisher('cmd_vel', Twist, queue_size=1)

        self.pwm_state_sub = rospy.Subscriber(
            '/mccd/status', Status, self.status_cb)

        stdscr.clear()
        stdscr.addstr(0, 0, "Waiting for /mccd/enable_pwm service...")
        stdscr.refresh()

        rospy.wait_for_service('/mccd/enable_pwm')
        self.change_pwm = rospy.ServiceProxy('/mccd/enable_pwm', SetBool)

        self.joySubscriber = rospy.Subscriber('joy', Joy, self.joyCallback)

        self.lastTimeUpdate = time.time()
        self.minUpdatePeriod = 0.3  # resend data every x sec.

        while not rospy.is_shutdown():
            if self.forceQuit == True:
                break

            if self.update_info:
                self.print_info(stdscr)

            t = time.time()

            if self.lastTimeUpdate + self.minUpdatePeriod < t:
                self.send_vel(self.lin_speed_cmd, self.ang_speed_cmd)
                self.lastTimeUpdate = t

        curses.endwin()
        rospy.loginfo("Shutting down joy_teleop")

    def print_info(self, stdscr):

        stdscr.clear()

        stdscr.addstr(0, 0, """
         JOY TELEOP        _
            ,---.          U
           ;     \         ;
       .==\\"/==.  `-.___.-'
      ((+) .  .:)
      |`.-(o)-.'|
      \/  \_/  \/
        """)

        stdscr.addstr(9, 0, "{:^10} {:^10s} {:^10s}".format(
            "LEFT JOY", "START", "PS"))
        stdscr.addstr(10, 0, "{:^10s} {:^10s} {:^10s}".format(
            "Speed", "Toggle PWM", "Shutdown"))

        curses.init_pair(1, curses.COLOR_WHITE, curses.COLOR_GREEN)

        stdscr.addstr(12, 0, "Status:")
        stdscr.addstr(12, 12, self.status_to_string(
            self.status), curses.A_BOLD | curses.color_pair(1))

        lin_speed_str = "{:.3f}".format(self.lin_speed_cmd)
        ang_speed_str = "{:.3f}".format(self.ang_speed_cmd)

        stdscr.addstr(13, 0, "Lin. Vel.:")
        stdscr.addstr(
            13, 12, lin_speed_str, curses.A_BOLD | curses.color_pair(1))

        stdscr.addstr(14, 0, "Ang. Vel.:")
        stdscr.addstr(
            14, 12, ang_speed_str, curses.A_BOLD | curses.color_pair(1))

        stdscr.refresh()

        self.update_info = False

    def status_cb(self, msg):
        if msg.status != self.status:
            self.status = msg.status
            self.update_info = True

    def status_to_string(self, pwm_state):
        if pwm_state == 0:
            return "OFF"
        elif pwm_state == 1:
            return "ON"
        elif pwm_state == 2:
            return "OVERCURRENT"
        elif pwm_state == 3:
            return "CMD_VEL_TIMEOUT"
        else:
            return "UNKNOWN"

    def send_vel(self, lin_vel, ang_vel):
        twist = Twist()
        twist.linear.x = lin_vel
        twist.angular.z = ang_vel
        self.publisher.publish(twist)

    def enablePwm(self, enabled):

        try:
            self.change_pwm(enabled)

        except rospy.ServiceException as e:
            print("Service call failed: %s" % e)


def main(stdscr):

    stdscr.clear()

    rospy.init_node("joy_teleop")
    try:
        JoyTeleopNode(stdscr)
    except rospy.ROSInterruptException:
        pass


if __name__ == '__main__':
    curses.wrapper(main)
